[llvm-commits] [llvm] r54415 - in /llvm/trunk: lib/Transforms/Scalar/LoopStrengthReduce.cpp test/Transforms/LoopStrengthReduce/2008-08-06-CmpStride.ll

Evan Cheng evan.cheng at apple.com
Wed Aug 6 11:04:45 PDT 2008


Author: evancheng
Date: Wed Aug  6 13:04:43 2008
New Revision: 54415

URL: http://llvm.org/viewvc/llvm-project?rev=54415&view=rev
Log:
Fix PR2355: bug in ChangeCompareStride. When the loop termination compare is the only use of its iv stride, the stride can be eliminated by moving it to another stride. If the scale is negative, swap the predicate instead of using a inverse predicate.

Added:
    llvm/trunk/test/Transforms/LoopStrengthReduce/2008-08-06-CmpStride.ll
Modified:
    llvm/trunk/lib/Transforms/Scalar/LoopStrengthReduce.cpp

Modified: llvm/trunk/lib/Transforms/Scalar/LoopStrengthReduce.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Transforms/Scalar/LoopStrengthReduce.cpp?rev=54415&r1=54414&r2=54415&view=diff

==============================================================================
--- llvm/trunk/test/Transforms/LoopStrengthReduce/2008-08-06-CmpStride.ll (added)
+++ llvm/trunk/test/Transforms/LoopStrengthReduce/2008-08-06-CmpStride.ll Wed Aug  6 13:04:43 2008
@@ -0,0 +1,24 @@
+; RUN: llvm-as < %s | opt -loop-reduce | llvm-dis | grep ugt
+; PR2355
+
+ at .str = internal constant [4 x i8] c"%d\0A\00"
+
+declare i32 @printf(i8* noalias , ...) nounwind
+
+define i32 @main() nounwind {
+entry:
+        br label %forbody
+
+forbody:
+        %i.0 = phi i32 [ 0, %entry ], [ %inc, %forbody ]                ; <i32>[#uses=3]
+        %sub14 = sub i32 1027, %i.0             ; <i32> [#uses=1]
+        %mul15 = mul i32 %sub14, 10             ; <i32> [#uses=1]
+        %add166 = or i32 %mul15, 1              ; <i32> [#uses=1] *
+        call i32 (i8*, ...)* @printf( i8* noalias  getelementptr ([4 x i8]* @.str, i32 0, i32 0), i32 %add166 ) nounwind
+        %inc = add i32 %i.0, 1          ; <i32> [#uses=3]
+        %cmp = icmp ult i32 %inc, 1027          ; <i1> [#uses=1]
+        br i1 %cmp, label %forbody, label %afterfor
+
+afterfor:               ; preds = %forcond
+        ret i32 0
+}





More information about the llvm-commits mailing list
--- llvm/trunk/lib/Transforms/Scalar/LoopStrengthReduce.cpp (original)
+++ llvm/trunk/lib/Transforms/Scalar/LoopStrengthReduce.cpp Wed Aug  6 13:04:43 2008
@@ -1624,10 +1624,10 @@
         continue;
       }
 
       // If scale is negative, use inverse predicate unless it's testing
       // for equality.
       if (Scale < 0 && !Cond->isEquality())
-        Predicate = ICmpInst::getInversePredicate(Predicate);
+        Predicate = ICmpInst::getSwappedPredicate(Predicate);
 
       NewStride = &StrideOrder[i];
       break;
@@ -1624,10 +1624,10 @@
       }
 
-      // If scale is negative, use inverse predicate unless it's testing
+      // If scale is negative, use swapped predicate unless it's testing
       // for equality.
       if (Scale < 0 && !Cond->isEquality())

Added: llvm/trunk/test/Transforms/LoopStrengthReduce/2008-08-06-CmpStride.ll
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/test/Transforms/LoopStrengthReduce/2008-08-06-CmpStride.ll?rev=54415&view=auto

==============================================================================